const Evented = require('../lib/evented')

describe('Evented', () => {
  let emitter

  describe('on', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('registers a handler to a topic', () => {
      let callCount = 0
      emitter.on('topic', () => callCount++)
      emitter.emit('topic')
      expect(callCount).to.equal(1)
    })
  })

  describe('onDefault', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('registers a default handler to a topic', () => {
      let callCount = 0
      emitter.onDefault('topic', () => callCount++)
      emitter.emit('topic')
      expect(callCount).to.equal(1)
    })
  })

  describe('off', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('unregisters a handler from a topic', () => {
      let callCount = 0
      const handler = () => callCount++
      emitter.on('topic', handler)
      emitter.emit('topic')
      emitter.off('topic', handler)
      emitter.emit('topic')
      expect(callCount).to.equal(1)
    })
  })

  describe('once', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('registers a handler that only gets called once', () => {
      let callCount = 0
      emitter.once('topic', () => callCount++)
      emitter.emit('topic')
      emitter.emit('topic')
      expect(callCount).to.equal(1)
    })
  })

  describe('emit', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('calls all handlers for a topic', () => {
      let callCount = 0
      emitter.on('topic', () => callCount++)
      emitter.on('topic', () => callCount++)
      emitter.emit('topic')
      expect(callCount).to.equal(2)
    })
  })

  describe('hasEvent', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('checks if there are handlers for a topic', () => {
      emitter.on('topic', () => {})
      expect(emitter.hasEvent('topic')).to.be.true
      expect(emitter.hasEvent('anotherTopic')).to.be.false
    })
  })

  describe('size', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('counts the handlers for a topic', () => {
      emitter.on('topic', () => {})
      emitter.on('topic', () => {})
      expect(emitter.size('topic')).to.equal(2)
    })
  })

  describe('clear', () => {
    before(() => emitter = new Evented())
    after(() => emitter = null)

    it('clears all handlers from a topic', () => {
      let callCount = 0
      emitter.on('topic', () => callCount++)
      emitter.clear('topic')
      emitter.emit('topic')
      expect(callCount).to.equal(0)
    })

    it('clears all handlers when no topic is given', () => {
      let callCount = 0
      emitter.on('topic', () => callCount++)
      emitter.on('anotherTopic', () => callCount++)
      emitter.clear()
      emitter.emit('topic')
      emitter.emit('anotherTopic')
      expect(callCount).to.equal(0)
    })
  })
})
